local max = math.max
local CreateFrame = CreateFrame
local GREEN_FONT_COLOR = GREEN_FONT_COLOR
local PlaySound = PlaySound
local Talented = Talented
local GameTooltip = GameTooltip

local L = LibStub("AceLocale-3.0"):GetLocale("Talented")

Talented.uielements = {}
local function MakeButton(parent)
	local button = CreateFrame("Button", nil, parent)
	button:SetNormalFontObject(GameFontNormal)
	button:SetHighlightFontObject(GameFontHighlight)
	button:SetDisabledFontObject(GameFontDisable)

	local CreateTexture = function(path, blend)
		local t = button:CreateTexture()
		if path then t:SetTexture(path) end
		if blend then t:SetBlendMode(blend) end
		t:SetTexCoord(0, 0.625, 0, 0.6875)
		t:SetAllPoints(button)
		return t
	end

	button:SetNormalTexture(CreateTexture("Interface\\Buttons\\UI-Panel-Button-Up"))
	button:SetPushedTexture(CreateTexture("Interface\\Buttons\\UI-Panel-Button-Down"))
	button:SetDisabledTexture(CreateTexture("Interface\\Buttons\\UI-Panel-Button-Disabled"))
	button:SetHighlightTexture(CreateTexture("Interface\\Buttons\\UI-Panel-Button-Highlight", "ADD"))

	table.insert(Talented.uielements, button)
	return button
end

local function CreateBaseButtons(parent)
	local function Frame_OnEnter(self)
		if self.tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
			GameTooltip:SetText(self.tooltip, nil, nil, nil, nil, 1)
		end
	end

	local function Frame_OnLeave(self)
		if GameTooltip:IsOwned(self) then
			GameTooltip:Hide()
		end
	end

	local b = MakeButton(parent)
	parent.bactions = b

	b:SetText(L["Actions"])
	local width = max(110, b:GetTextWidth() + 22)
	b:SetWidth(width)
	b:SetHeight(22)
	b:SetScript("OnClick", function (self)
		Talented:OpenActionMenu(self)
	end)
	b:SetPoint("TOPLEFT", 14, -4)

	b = MakeButton(parent)
	parent.bmode = b

	b:SetText(L["Templates"])
	width = max(110, b:GetTextWidth() + 22)
	b:SetWidth(width)
	b:SetHeight(22)
	b:SetScript("OnClick", function (self)
		Talented:OpenTemplateMenu(self)
	end)
	b:SetPoint("LEFT", parent.bactions, "RIGHT", 14, 0)


	local e = CreateFrame("EditBox", nil, parent)
	parent.editname = e
	e:SetFontObject(ChatFontNormal)
	e:SetTextColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b)
	width = 400
	e:SetWidth(width)
	e:SetHeight(13)
	e:SetAutoFocus(false)

	e:SetScript("OnEscapePressed", function (this)
			this:ClearFocus()
		end)
	e:SetScript("OnEditFocusLost", function (this)
			this:SetText(Talented.template.name)
		end)
	e:SetScript("OnEnterPressed",  function (this)
			Talented:UpdateTemplateName(Talented.template, this:GetText())
			Talented:UpdateView()
			this:ClearFocus()
		end)
	e:SetScript("OnEnter", Frame_OnEnter)
	e:SetScript("OnLeave", Frame_OnLeave)
	e:SetPoint("LEFT", parent.bmode, "RIGHT", 14, 0)
	e.tooltip = L["You can edit the name of the template here. You must press the Enter key to save your changes."]

	local fs = parent:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	fs:SetJustifyH("LEFT")
	fs:SetWidth(width)
	fs:SetHeight(13)
	fs:SetPoint("LEFT", parent.bmode, "RIGHT", 14, 0)
	parent.targetname = fs

	do
		local f = CreateFrame("Frame", nil, parent)
		local fs = f:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		fs:SetJustifyH("RIGHT")
		fs:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -14, 8)
		f:SetPoint("BOTTOMRIGHT")
		f:SetFrameLevel(parent:GetFrameLevel()+2)
		f.text = fs
		parent.pointsleft = f
	end

	local cb = CreateFrame("Checkbutton", nil, parent)
	parent.checkbox = cb

	local makeTexture = function (path, blend)
		local t = cb:CreateTexture()
		t:SetTexture(path)
		t:SetAllPoints(cb)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	cb:SetHeight(20)
	cb:SetWidth(20)

	local fs = cb:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
	cb.label = fs
	fs:SetJustifyH("LEFT")
	fs:SetWidth(400)
	fs:SetHeight(20)
	fs:SetPoint("LEFT", cb, "RIGHT", 1, 1)
	cb:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Up"))
	cb:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Down"))
	cb:SetDisabledTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled"))
	cb:SetCheckedTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Check"))
	cb:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-CheckBox-Highlight", "ADD"))
	cb:SetScript("OnClick", function ()
		if Talented.mode == "edit" then
			Talented:SetMode("view")
		else
			Talented:SetMode("edit")
		end
	end)
	cb.label:SetText(L["Edit talents"])
	cb:SetScript("OnEnter", Frame_OnEnter)
	cb:SetScript("OnLeave", Frame_OnLeave)
	cb.tooltip = L["Toggle editing of talents."]
	cb:SetPoint("BOTTOMLEFT", parent, "BOTTOMLEFT", 14, 8)
	cb:SetFrameLevel(parent:GetFrameLevel()+2)
	if select(2, UnitClass"player") == "HUNTER" then
		local pet = MakeButton(parent)
		parent.bpet = pet
		pet:SetText(L["Pet"])
		width = max(55, pet:GetTextWidth() + 22)
		pet:SetWidth(width)
		pet:SetHeight(22)
		pet:SetPoint("LEFT", parent.bmode, "RIGHT", 14, 0)
		parent.targetname:SetPoint("LEFT", pet, "RIGHT", 14, 0)
		parent.editname:SetPoint("LEFT", pet, "RIGHT", 14, 0)

		function Talented:UNIT_PET(_, unit)
			if unit == "player" then
				if select(2, HasPetUI()) then
					self.base.bpet:Enable()
				else
					self.base.bpet:Disable()
				end
			end
		end
		pet:SetScript("OnShow", function (self)
			if select(2, HasPetUI()) then
				self:Enable()
			else
				self:Disable()
			end
		end)
		Talented:RegisterEvent"UNIT_PET"

		if Talented.PET_TALENT_POINTS_CHANGED then
			Talented:RegisterEvent"PET_TALENT_POINTS_CHANGED"
		end

		pet:SetScript("OnClick", function (self)
			if not select(2, HasPetUI()) then return end
			if not PlayerTalentFrameType1 then
				LoadAddOn("Blizzard_TalentUI")
			end
			if not PlayerTalentFrameType1.talented_hook then
				PlayerTalentFrameType1.talented_hook = PlayerTalentFrameType1:GetScript("OnClick")
				PlayerTalentFrameType1:SetScript("OnClick", function (self)
					HideUIPanel(PlayerTalentFrame)
					Talented:ToggleTalentFrame()
				end)
			end
			local function new_OnClick()
				HideUIPanel(parent)
				ShowUIPanel(PlayerTalentFrame)
				PlayerTalentFrameType_OnClick(PlayerTalentFrameType2)
			end
			self:SetScript("OnClick", new_OnClick)
			return new_OnClick()
		end)
	end
end

function Talented:CreateBaseFrame()
	local onshow = function ()
		Talented:RegisterEvent("CHARACTER_POINTS_CHANGED")
		Talented:RegisterEvent("MODIFIER_STATE_CHANGED")
		SetButtonPulse(TalentMicroButton, 0, 1)
		PlaySound("TalentScreenOpen")
		UpdateMicroButtons()
	end

	local frame = CreateFrame("Frame", "TalentedFrame", UIParent)
	frame:SetFrameStrata("DIALOG")
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetWidth(50)
	frame:SetHeight(50)
	frame:SetBackdrop({
		bgFile = "Interface\\TutorialFrame\\TutorialFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		edgeSize = 16,
		tileSize = 32,
		insets = {
			left = 5,
			right = 5,
			top = 5,
			bottom = 5
		}
	})

	frame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
	frame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() Talented:SaveFramePosition() end)

	frame.points = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	frame.points:SetJustifyH("RIGHT")
	frame.points:SetWidth(80)
	frame.points:SetHeight(14)
	frame.points:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -40, -6)

	local close = CreateFrame("Button", nil, frame)
	frame.close = close
	table.insert(Talented.uielements, close)

	local makeTexture = function (path, blend)
		local t = close:CreateTexture()
		t:SetAllPoints(close)
		t:SetTexture(path)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	close:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up"))
	close:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down"))
	close:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD"))
	close:SetScript("OnClick", function () HideUIPanel(frame) end)

	close:SetWidth(32)
	close:SetHeight(32)
	close:ClearAllPoints()
	close:SetPoint("TOPRIGHT", 1, 0)

	CreateBaseButtons(frame)

	frame:SetScript("OnShow", onshow)
	frame:SetScript("OnHide", function()
			PlaySound("TalentScreenClose")
			if Talented.mode == "apply" then
				Talented:SetMode(Talented:GetDefaultMode())
				Talented:Print(L["Error! Talented window has been closed during template application. Please reapply later."])
				Talented:EnableUI()
			end
			Talented:UnregisterEvent("CHARACTER_POINTS_CHANGED")
			Talented:UnregisterEvent("MODIFIER_STATE_CHANGED")
			Talented:CloseMenu()
		end)

	UIPanelWindows["TalentedFrame"] = { area = "center", pushable = 0, whileDead = 1 }

	self.base = frame
	if self.UNIT_PET then
		self:UNIT_PET(nil, "player")
	end
	onshow()
	frame:Hide()

	if not self:IsTemplateMode() then
		frame.editname:Hide()
	end

	self.CreateBaseFrame = function () return self.base end
	return frame
end

function Talented:EnableUI(enable)
	if enable then
		for _, element in ipairs(self.uielements) do
			element:Enable()
		end
	else
		for _, element in ipairs(self.uielements) do
			element:Disable()
		end
	end
end
